-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local persistencyfile = 'settings/bluetac/settings.ini'

local M = {}

local settingsList = {       -- default value
  lookaheadReverseCam = true,
  lookaheadSensitivity = 0.05,
  lookaheadLinearity = 1000
}

local defaultValues = { }
for k,v in pairs(settingsList) do
  defaultValues[k] = v
end

local values = deepcopy(defaultValues)
local lastSavedTime = 0

local function sendUIState()
  log("D", "settings.sendUIState", dumps(values))
  guihooks.trigger('SettingsChanged', {values = values})
end


local function save()
  --log("D", "settings", "Saving options to disk")
  lastSavedTime = os.clock()

  -- save options
  -- log("D", "settings.save", dumps(values))
  local localValues = {}
  local cloudValues = {}
  for k, v in pairs(values) do
      localValues[k] = values[k]
  end
  -- log("D", "settings.save_local", dumps(localValues))
  saveIni(persistencyfile, localValues)

  -- TorqueScript.eval( 'saveGlobalOptions();' );
  -- let UI and Lua know
  sendUIState()
  extensions.hook('onSettingsChanged')
  be:queueAllObjectLua('onSettingsChanged()')
end

local function setState(newState, ignoreCache)
  if newState == nil then return end
  local isChanged = false
  for k, s in pairs(newState) do
    if values[k] == nil or (tostring(s) ~= tostring(values[k]))  then
      isChanged = true
      values[k] = s
    end
  end

  if not isChanged and not ignoreCache then return end
  save()
end

local function setValue(key, value, ignoreCache)
  local newValues = deepcopy(values)
  newValues[key] = value
  setState(newValues, ignoreCache)
end

local function getValue(key, defaultValue)
  if values[key] == nil then
    return defaultValue
  end
  return values[key]
end

local function loadSettingValues()
  local data = loadIni(persistencyfile) or {}
  -- log("D", "settings.loadSettingValues_local", dumps(data))

  return data
end

local function load(ignoreCache)
  local newState = deepcopy(values)
  local data = loadSettingValues()
  if data then
    tableMerge(newState, defaultValues)
    tableMerge(newState, data)
  end

  -- log("D", "settings.load", dumps(newState))
  setState(newState, ignoreCache)
  -- dump(data)
end

local function reload()
  save()
end

local function initSettings()
  -- load the persistency file at least
  local data = loadSettingValues()
  if data then
    tableMerge(values, data)
  end
end

local function onFirstUpdate()
  -- force application of all settings the first time, since init() has not correctly applied all of them
  -- we could make init() call load(), but that would fail because it's still too early, and some stuff is not initialized yet
  load(true)
end

local function onFileChanged(filename, type)
  if filename == persistencyfile and (os.clock()-lastSavedTime) > 5 then
    load(false)
  end
end

local function onExtensionLoaded()
    log("I", "driveAhead", "init bluetac setting storage")
    initSettings()
end

M.onExtensionLoaded = onExtensionLoaded
M.onFirstUpdate = onFirstUpdate
M.onFileChanged = onFileChanged
M.requestState = sendUIState
M.setState = setState
M.setValue = setValue
M.getValue = getValue
M.reload = reload
M.save = save
M.load = load
M.initSettings = initSettings

return M
